"""
    AEMET API key saven in a file as module to import when needed but not
    upload into GitHub repository
"""


apikey = ("eyJhbGciOiJIUzI1NiJ9.eyJzdWIiOiJqYWltZWR1c2ViYWxsZXpAZ21haWwuY29tIi"
          + "wianRpIjoiOWVlZDEwMzMtZDRjMS00OWZiLThiYzAtZDc1Njk2MDhiY2JiIiwiaXN"
          + "zIjoiQUVNRVQiLCJpYXQiOjE2MTIxODI3NzgsInVzZXJJZCI6IjllZWQxMDMzLWQ0"
          + "YzEtNDlmYi04YmMwLWQ3NTY5NjA4YmNiYiIsInJvbGUiOiIifQ.Iidp4Sxk5khlZ8"
          + "mhd_JMyvdqJhZ37JyZjXX_wbO0mAM")
