""" Docstring """

__author__ = "Jaime Diez Gonzalez-Pardo"
__author_orcid__ = "0000-0001-7268-9933"
__date__ = "November, 2021"

__license__ = "GPL-3.0 License"
__version__ = "1.0.1"
__maintainer__ = "Jaimedgp"
__email__ = "jaime.diez.gp@gmail.com"

__repository__ = "https://github.com/Jaimedgp/pyAEMET"
__dataset_doi__ = "10.5281/zenodo.5642979"

from pyaemet.valores_climatologicos import AemetClima
